/****************************************************************************\
*                                                                            *
*  FIRST.CPP                                                                 *
*                                                                            *
*  This is a C++ version of the first Fastgraph for Windows example program. *
*  It illustrates how one might set up and use a virtual buffer class.       *
*                                                                            *
\****************************************************************************/

#include <fgwin.h>

long WINAPI _export WindowProc(HWND,UINT,UINT,LONG);

/****************************************************************************\
*                                                                            *
*  Virtual Buffer class and function declarations                            *
*                                                                            *
\****************************************************************************/

class VB {
private:
   int hvb;                             // virtual buffer handle
public:
   VB(int, int, int init=FALSE);        // constructor function
   ~VB();                               // destructor function
   int handle();                        // member function (get VB handle)
};

VB::VB(int width, int height, int init) // constructor
{
   if (init) fg_vbinit();
   hvb = fg_vballoc(width,height);
   fg_vbopen(hvb);
   fg_vbcolors();
}

VB::~VB()                               // destructor
{
   fg_vbclose();
   fg_vbfree(hvb);
   fg_vbfin();
}

int VB::handle(void)                    // member function to get the VB handle
{
   return(hvb);
}

/****************************************************************************\
*                                                                            *
*  WinMain                                                                   *
*                                                                            *
\****************************************************************************/

int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPSTR lpszCmdParam, int nCmdShow)
{
   static char szAppName[] = "FGfirst";
   HWND        window;
   MSG         msg;
   WNDCLASS    wndclass;

   if (!hPrevInstance)
   {
      wndclass.style         = CS_HREDRAW | CS_VREDRAW;
      wndclass.lpfnWndProc   = WindowProc;
      wndclass.cbClsExtra    = 0;
      wndclass.cbWndExtra    = 0;
      wndclass.hInstance     = hInstance;
      wndclass.hIcon         = LoadIcon(NULL,IDI_APPLICATION);
      wndclass.hCursor       = LoadCursor(NULL,IDC_ARROW);
      wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
      wndclass.lpszMenuName  = NULL;
      wndclass.lpszClassName = szAppName;
      RegisterClass (&wndclass);
   }

   window = CreateWindow(szAppName,          // window class name
      "First Fastgraph for Windows Program", // window caption
      WS_OVERLAPPEDWINDOW,     // window style
      CW_USEDEFAULT,           // initial x position
      CW_USEDEFAULT,           // initial y position
      CW_USEDEFAULT,           // initial x size
      CW_USEDEFAULT,           // initial y size
      NULL,                    // parent window handle
      NULL,                    // window menu handle
      hInstance,               // program instance handle
      NULL);                   // creation parameters

   VB vb(320,200,TRUE);        // create 320x200 virtual buffer

   ShowWindow(window,nCmdShow);
   UpdateWindow(window);

   while (GetMessage(&msg,NULL,0,0))
   {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }
   return msg.wParam;
}

/****************************************************************************\
*                                                                            *
*  WindowProc                                                                *
*                                                                            *
*  Window procedure to handle messages sent to the window.                   *
*                                                                            *
\****************************************************************************/

HDC      hdc;
HPALETTE hpal;
UINT     cxClient, cyClient;

long WINAPI _export WindowProc(HWND window, UINT message,
                               UINT wParam, LONG lParam)
{
   PAINTSTRUCT ps;

   switch (message)
   {
      case WM_CREATE:
         hdc = GetDC(window);
         fg_setdc(hdc);
         hpal = fg_defpal();
         fg_realize(hpal);
         return 0;

      case WM_PAINT:
         BeginPaint(window,&ps);
         fg_vbscale(0,fg_getmaxx(),0,fg_getmaxy(),0,cxClient-1,0,cyClient-1);
         EndPaint(window,&ps);
         return 0;

      case WM_SETFOCUS:
         fg_realize(hpal);
         InvalidateRect(window,NULL,TRUE);
         return 0;

      case WM_SIZE:
         cxClient = LOWORD(lParam);
         cyClient = HIWORD(lParam);
         return 0;

      case WM_DESTROY:
         DeleteObject(hpal);
         ReleaseDC(window,hdc);
         PostQuitMessage(0);
         return 0;
   }
   return DefWindowProc(window,message,wParam,lParam);
}
